unit native;

{$mode objfpc}{$H+}


{$COPERATORS ON}
{.$INLINE ON}

{$MODESWITCH ADVANCEDRECORDS}

interface

uses
  Classes, SysUtils, GLZVectorMath;

type

  { TNativeGLZVector4f }

  TNativeGLZVector4f =  record  // With packed record the performance decrease a little
  public
    procedure Create(Const aX,aY,aZ: single; const aW : Single = 0); overload;
    procedure Create(Const anAffineVector: TGLZVector3f; const aW : Single = 0); overload;

    function ToString : String;

    class operator +(constref A, B: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    class operator -(constref A, B: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    class operator *(constref A, B: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    class operator /(constref A, B: TNativeGLZVector4f): TNativeGLZVector4f; overload;

    class operator +(constref A: TNativeGLZVector4f; constref B:Single): TNativeGLZVector4f; overload;
    class operator -(constref A: TNativeGLZVector4f; constref B:Single): TNativeGLZVector4f; overload;
    class operator *(constref A: TNativeGLZVector4f; constref B:Single): TNativeGLZVector4f; overload;
    class operator /(constref A: TNativeGLZVector4f; constref B:Single): TNativeGLZVector4f; overload;

    class operator -(constref A: TNativeGLZVector4f): TNativeGLZVector4f; overload;

    class operator =(constref A, B: TNativeGLZVector4f): Boolean;
    class operator >=(constref A, B: TNativeGLZVector4f): Boolean;
    class operator <=(constref A, B: TNativeGLZVector4f): Boolean;
    class operator >(constref A, B: TNativeGLZVector4f): Boolean;
    class operator <(constref A, B: TNativeGLZVector4f): Boolean;
    class operator <>(constref A, B: TNativeGLZVector4f): Boolean;

    function Shuffle(const x,y,z,w : Byte):TNativeGLZVector4f;
    function MinXYZComponent : Single;
    function MaxXYZComponent : Single;

    function Abs:TNativeGLZVector4f;overload;
    function Negate:TNativeGLZVector4f;
    function  DivideBy2:TNativeGLZVector4f;
    function Distance(constref A: TNativeGLZVector4f):Single;
    function Length:Single;
    function DotProduct(constref A: TNativeGLZVector4f):Single;
    function CrossProduct(constref A: TNativeGLZVector4f): TNativeGLZVector4f;
    function Normalize: TNativeGLZVector4f;
    function Norm:Single;
    function Min(constref B: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    function Min(constref B: Single): TNativeGLZVector4f; overload;
    function Max(constref B: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    function Max(constref B: Single): TNativeGLZVector4f; overload;
    function Clamp(Constref AMin, AMax: TNativeGLZVector4f): TNativeGLZVector4f; overload;
    function Clamp(constref AMin, AMax: Single): TNativeGLZVector4f; overload;
    function MulAdd(Constref B, C: TNativeGLZVector4f): TNativeGLZVector4f;
    function MulDiv(Constref B, C: TNativeGLZVector4f): TNativeGLZVector4f;

    function Lerp(Constref B: TNativeGLZVector4f; Constref T:Single): TNativeGLZVector4f;
    function AngleCosine(constref A : TNativeGLZVector4f): Single;
    function AngleBetween(Constref A, ACenterPoint : TNativeGLZVector4f): Single;

    function Combine(constref V2: TNativeGLZVector4f; constref F1: Single): TNativeGLZVector4f;
    function Combine2(constref V2: TNativeGLZVector4f; const F1, F2: Single): TNativeGLZVector4f;
    function Combine3(constref V2, V3: TNativeGLZVector4f; const F1, F2, F3: Single): TNativeGLZVector4f;

    function Perpendicular(constref N : TNativeGLZVector4f) : TNativeGLZVector4f;

    function Reflect(constref N: TNativeGLZVector4f): TNativeGLZVector4f;

//    function MoveAround(constRef AMovingUp, ATargetPosition: TNativeGLZVector4f; pitchDelta, turnDelta: Single): TNativeGLZVector4f;

    procedure pAdd(constref A: TNativeGLZVector4f);overload;
    procedure pSub(constref A: TNativeGLZVector4f);overload;
    procedure pMul(constref A: TNativeGLZVector4f);overload;
    procedure pDiv(constref A: TNativeGLZVector4f);overload;
    procedure pAdd(constref A: Single);overload;
    procedure pSub(constref A: Single);overload;
    procedure pMul(constref A: Single);overload;
    procedure pDiv(constref A: Single);overload;
    procedure pInvert;
    procedure pNegate;
    procedure pAbs;
    procedure pDivideBy2;
    procedure pCrossProduct(constref A: TNativeGLZVector4f);
    procedure pNormalize;
    procedure pMin(constref B: TNativeGLZVector4f); overload;
    procedure pMin(constref B: Single);overload;
    procedure pMax(constref B: TNativeGLZVector4f); overload;
    procedure pMax(constref B: Single); overload;
    procedure pClamp(Constref AMin, AMax: TNativeGLZVector4f); overload;
    procedure pClamp(constref AMin, AMax: Single); overload;
    procedure pMulAdd(Constref B, C: TNativeGLZVector4f); // (Self*B)+c
    procedure pMulDiv(Constref B, C: TNativeGLZVector4f); // (Self*B)-c
    //procedure Lerp(Constref B: TNativeGLZVector4f; Constref T:Single): TNativeGLZVector4f;


    case Byte of
      0: (V: TGLZVector4fType);
      1: (X, Y, Z, W: Single);
      2: (AsVector3f : TGLZVector3f);
  end;


  function Compare(constref A: TNativeGLZVector4f; constref B: TGLZVector4f;Espilon: Single = 1e-10): boolean;
  function IsEqual(A,B: Single; Epsilon: single = 1e-10): boolean; inline;

implementation
uses
  GLZMath;

function IsEqual(A,B: Single; Epsilon: single): boolean;
begin
  Result := Abs(A-B) < Epsilon;
end;

procedure TNativeGLZVector4f.Create(const aX, aY, aZ: single; const aW: Single);
begin
   Self.X := AX;
   Self.Y := AY;
   Self.Z := AZ;
   Self.W := AW;
end;

procedure TNativeGLZVector4f.Create(const anAffineVector: TGLZVector3f;
  const aW: Single);
begin
   Self.X := anAffineVector.X;
   Self.Y := anAffineVector.Y;
   Self.Z := anAffineVector.Z;
   Self.W := AW;
end;

function TNativeGLZVector4f.ToString : String;
begin
   Result := '(X: '+FloattoStrF(Self.X,fffixed,5,5)+
            ' ,Y: '+FloattoStrF(Self.Y,fffixed,5,5)+
            ' ,Z: '+FloattoStrF(Self.Z,fffixed,5,5)+
            ' ,W: '+FloattoStrF(Self.W,fffixed,5,5)+')';
End;


function TNativeGLZVector4f.Shuffle(const x,y,z,w : Byte):TNativeGLZVector4f;
begin
  Result.V[x]:=Self.X;
  Result.V[y]:=Self.Y;
  Result.V[z]:=Self.Z;
  Result.V[w]:=Self.W;
End;

function TNativeGLZVector4f.MinXYZComponent : Single;
begin
   Result:=Min3s(Self.X, Self.Y, Self.Z);
end;

function TNativeGLZVector4f.MaxXYZComponent : Single;
begin
   Result:=Max3s(Self.X, Self.Y, Self.Z);
end;


{$i native.inc}



function Compare(constref A: TNativeGLZVector4f; constref B: TGLZVector4f;
  Espilon: Single): boolean;
begin
  Result := true;
  if not IsEqual (A.X, B.X, Espilon) then Result := False;
  if not IsEqual (A.Y, B.Y, Espilon) then Result := False;
  if not IsEqual (A.Z, B.Z, Espilon) then Result := False;
  if not IsEqual (A.W, B.W, Espilon) then Result := False;
end;





end.
