unit TestCase1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry,
  native, GLZVectorMath;

type

  { TTestCase1 }

  { TTestOperators }

  TTestOperators = class(TTestCase)
  protected
   procedure Setup; override;
  public
   {$CODEALIGN RECORDMIN=16}
   vt1,vt2, vt3 : TGLZVector4f;
   at1, at2, at3, at4: TGLZVector;
   nt1,nt2, nt3 : TNativeGLZVector4f;
   ant1,ant2, ant3, ant4 : TNativeGLZVector4f;
   {$CODEALIGN RECORDMIN=4}
   Fs1,Fs2 : Single;
   nb, vb: boolean;

  published
   procedure TestCompare;       // these two test ensure we have data to play
   procedure TestCompareFalse;  // with and tests the compare function.
   procedure TestAddVector;
   procedure TestAddSingle;
   procedure TestSubVector;
   procedure TestSubSingle;
   procedure TestMulVector;
   procedure TestMulSingle;
   procedure TestDivVector;
   procedure TestDivSingle;
   procedure TestEqualVector;
   procedure TestSomeEqualVector;
   procedure TestUnEqualVector;
   procedure TestLTorEqualVector;
   procedure TestLTorEqualOneGreaterVector;
   procedure TestGTorEqualVector;
   procedure TestGTorEqualOneLessVector;
   procedure TestLTVector;
   procedure TestLTOneGreaterVector;
   procedure TestGTVector;
   procedure TestGTOneLessVector;
  end;

implementation

procedure TTestOperators.Setup;
begin
  vt1.Create(5.850,-15.480,8.512,1.5);
  vt2.Create(1.558,6.512,4.525,1.0);
  nt1.V := vt1.V;
  nt2.V := vt2.V;
  Fs1 := 1.5;
  Fs2 := 5.5;
end;



{Test IsEqual and we have same values for each class ttpe}
procedure TTestOperators.TestCompare;
begin
  AssertTrue('Test Values do not match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt1,vt1));
end;

procedure TTestOperators.TestCompareFalse;
begin
  AssertFalse('Test Values should not match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt1,vt2));
end;

procedure TTestOperators.TestAddVector;
begin
  nt3 := nt2 + nt1;
  vt3 := vt1 + vt2;
  AssertTrue('Vector + Vector no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestAddSingle;
begin
  nt3 := nt1 + fs1;
  vt3 := vt1 + fs1;
  AssertTrue('Vector + Single no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestSubVector;
begin
  nt3 := nt2 - nt1;
  vt3 := vt2 - vt1;
  AssertTrue('Vector - Vector no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestSubSingle;
begin
  nt3 := nt1 - fs1;
  vt3 := vt1 - fs1;
  AssertTrue('Vector - Single no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestMulVector;
begin
  nt3 := nt2 * nt1;
  vt3 := vt2 * vt1;
  AssertTrue('Vector x Vector no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestMulSingle;
begin
  nt3 := nt1 * fs1;
  vt3 := vt1 * fs1;
  AssertTrue('Vector x Single no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestDivVector;
begin
  nt3 := nt2 / nt1;
  vt3 := vt2 / vt1;
  AssertTrue('Vector / Vector no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestDivSingle;
begin
  nt3 := nt1 / fs1;
  vt3 := vt1 / fs1;
  AssertTrue('Vector / Single no match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestOperators.TestEqualVector;
begin
  nb := nt1 = nt1;
  vb := vt1 = vt1;
  AssertTrue('Vectors should be equal'+nt1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestSomeEqualVector;
begin
  at1.V := vt1.V;
  at1.X := at1.X +1;
  at1.Z := at1.Z - 1;
  ant1.V := at1.V;
  nb := nt1 = ant1;
  vb := vt1 = at1;
  AssertTrue('Vectors should not be equal'+vt1.ToString+' --> '+at1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestUnEqualVector;
begin
  nb := nt1 = nt2;
  vb := vt1 = vt2;
  AssertTrue('Vectors should be equal'+vt1.ToString+' --> '+vt2.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestLTorEqualVector;
begin
  at1 := vt1 - 1;
  ant1.V := at1.V;
  nb := at1 <= vt1;
  vb := ant1 <= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));

end;

procedure TTestOperators.TestLTorEqualOneGreaterVector;
begin
  at1 := vt1 - 1;
  at1.Y := vt1.Y + 1;
  ant1.V := at1.V;
  nb := at1 <= vt1;
  vb := ant1 <= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestGTorEqualVector;
begin
  at1 := vt1 + 1;
  ant1.V := at1.V;
  nb := at1 >= vt1;
  vb := ant1 >= nt1;
  AssertTrue('Vectors should be gt equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestGTorEqualOneLessVector;
begin
  at1 := vt1 + 1;
  at1.Y := vt1.Y - 1;
  ant1.V := at1.V;
  nb := at1 >= vt1;
  vb := ant1 >= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestLTVector;
begin
  at1 := vt1 - 1;
  ant1.V := at1.V;
  nb := at1 <= vt1;
  vb := ant1 <= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestLTOneGreaterVector;
begin
  at1 := vt1 - 1;
  at1.Y := vt1.Y + 1;
  ant1.V := at1.V;
  nb := at1 <= vt1;
  vb := ant1 <= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestGTVector;
begin
  at1 := vt1 + 1;
  ant1.V := at1.V;
  nb := at1 >= vt1;
  vb := ant1 >= nt1;
  AssertTrue('Vectors should be gt equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;

procedure TTestOperators.TestGTOneLessVector;
begin
  at1 := vt1 + 1;
  at1.Y := vt1.Y - 1;
  ant1.V := at1.V;
  nb := at1 >= vt1;
  vb := ant1 >= nt1;
  AssertTrue('Vectors should be equal'+at1.ToString+' --> '+vt1.ToString, not(nb xor vb));
end;


initialization
  RegisterTest(TTestOperators);
end.

