unit vectornumerics;
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry,
  native, GLZVectorMath;

type

  { TTestNumerics }

  TTestNumerics = class(TTestCase)
    protected
      procedure Setup; override;
    public
     {$CODEALIGN RECORDMIN=16}
     vt1,vt2, vt3 : TGLZVector4f;
     at1, at2, at3, at4, vorg: TGLZVector4f;
     nt1,nt2, nt3 : TNativeGLZVector4f;
     ant1,ant2, ant3, ant4, norg : TNativeGLZVector4f;
     {$CODEALIGN RECORDMIN=4}
     Fs1,Fs2 : Single;
     nb, vb: boolean;

    published
      procedure TestCompare;       // these two test ensure we have data to play
      procedure TestCompareFalse;  // with and tests the compare function.
      procedure TestAbsVector;
      procedure TestNegateVector;
      procedure TestDivideBy2Vector;
      procedure TestDistanceVector;
      procedure TestLengthVector;
      procedure TestDotProductVector;
      procedure TestCrossProductVector;
      procedure TestNormalizeVector;
      procedure TestNormVector;
      procedure TestMulAdd;
      procedure TestMulDiv;
      procedure TestpAddVector;
      procedure TestpAddSingle;
      procedure TestpSubVector;
      procedure TestpSubSingle;
      procedure TestpMulVector;
      procedure TestpMulSingle;
      procedure TestpDivVector;
      procedure TestpDivSingle;
      procedure TestpInvert;
      procedure TestpNegate;
      procedure TestpAbs;
      procedure TestpDivideBy2;
      procedure TestpCrossProduct;
      procedure TestpNormalize;
      procedure TestpMulAdd;
      procedure TestpMulDiv;
      procedure TestAngleCosine;
      //{$ifndef USE_ASM_AVX}
      procedure TestAngleBetweem;
      //{$endif}
  end;

implementation

procedure TTestNumerics.Setup;
begin
  vt1.Create(5.850,-15.480,8.512,1.5);
  vt2.Create(1.558,6.512,4.525,1.0);
  vorg.Create(0,0,0,0);
  norg.V := vorg.V;
  nt1.V := vt1.V;
  nt2.V := vt2.V;
  Fs1 := 1.5;
  Fs2 := 5.5;
end;

procedure TTestNumerics.TestCompare;
begin
  AssertTrue('Test Values do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestCompareFalse;
begin
  AssertFalse('Test Values should not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt2));
end;

procedure TTestNumerics.TestAbsVector;
begin
  nt3 := nt1.Abs;
  vt3 := vt1.Abs;
  AssertTrue('Vector Abs do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestNumerics.TestNegateVector;
begin
  nt3 := nt1.Negate;
  vt3 := vt1.Negate;
  AssertTrue('Vector negates do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestNumerics.TestDivideBy2Vector;
begin
  nt3 := nt1.DivideBy2;
  vt3 := vt1.DivideBy2;
  AssertTrue('Vector divide by 2 do not match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestNumerics.TestDistanceVector;
begin
  Fs1 := nt1.Distance(nt2);
  Fs2 := vt1.Distance(vt2);
  AssertTrue('Vector distances do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2));//
end;

procedure TTestNumerics.TestLengthVector;
begin
  Fs1 := nt1.Length;
  Fs2 := vt1.Length;
  AssertTrue('Vector lengths do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2,1e-5));
end;

procedure TTestNumerics.TestDotProductVector;
begin
  Fs1 := nt1.DotProduct(nt2);
  Fs2 := vt1.DotProduct(vt2);
  AssertTrue('Vector DotProducts do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2,1e-5));
end;

procedure TTestNumerics.TestCrossProductVector;
begin
  nt3 := nt1.CrossProduct(nt2);
  vt3 := vt1.CrossProduct(vt2);
  AssertTrue('Vector CrossProducts do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestNumerics.TestNormalizeVector;
begin
  nt3 := nt1.Normalize;
  vt3 := vt1.Normalize;
  AssertTrue('Vector Normalize do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3,1e-5));
end;

procedure TTestNumerics.TestNormVector;
begin
  Fs1 := nt1.Norm;
  Fs2 := vt1.Norm;
  AssertTrue('Vector Norms do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2,1e-4));
end;

procedure TTestNumerics.TestMulAdd;
begin
  nt3 := nt1.MulAdd(nt2, nt1);
  vt3 := vt1.MulAdd(vt2, vt1);
  AssertTrue('Vector MulAdds do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestNumerics.TestMulDiv;
begin
  nt3 := nt1.MulDiv(nt2, nt1);
  vt3 := vt1.MulDiv(vt2, vt1);
  AssertTrue('Vector pMulDivs do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;


procedure TTestNumerics.TestpAddVector;
begin
  nt1.pAdd(nt2);
  vt1.pAdd(vt2);
  AssertTrue('Vector pAdds Vectors do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpAddSingle;
begin
  nt1.pAdd(Fs1);
  vt1.pAdd(Fs1);
  AssertTrue('Vector pAdd Singles do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpSubVector;
begin
  nt1.pSub(nt2);
  vt1.pSub(vt2);
  AssertTrue('Vector pSub Vectors do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpSubSingle;
begin
  nt1.pSub(Fs1);
  vt1.pSub(Fs1);
  AssertTrue('Vector pSub singles do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpMulVector;
begin
  nt1.pMul(nt2);
  vt1.pMul(vt2);
  AssertTrue('Vector pMul Vectors do not match'+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpMulSingle;
begin
  nt1.pMul(Fs1);
  vt1.pMul(Fs1);
  AssertTrue('Vector pMul singles do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpDivVector;
begin
  nt1.pDiv(nt2);
  vt1.pDiv(vt2);
  AssertTrue('Vector pDiv Vectors do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpDivSingle;
begin
  nt1.pDiv(Fs1);
  vt1.pDiv(Fs1);
  AssertTrue('Vector pDiv singles do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpInvert;
begin
  nt1.pInvert;
  vt1.pInvert;
  AssertTrue('Vector pInverts do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpNegate;
begin
  nt1.pNegate;
  vt1.pNegate;
  AssertTrue('Vector pNegates do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpAbs;
begin
  nt1.pAbs;
  vt1.pAbs;
  AssertTrue('Vector pAbs do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpDivideBy2;
begin
  nt1.pDivideBy2;
  vt1.pDivideBy2;
  AssertTrue('Vector pAbs do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpCrossProduct;
begin
  nt1.pCrossProduct(nt2);
  vt1.pCrossProduct(vt2);
  AssertTrue('Vector pCrossProducts do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpNormalize;
begin
  nt1.pNormalize;
  vt1.pNormalize;
  AssertTrue('Vector pNormalizes do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1, 1e-5));
end;

procedure TTestNumerics.TestpMulAdd;
begin
  nt1.pMulAdd(nt2, nt1);
  vt1.pMulAdd(vt2, vt1);
  AssertTrue('Vector pMulAdds do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestpMulDiv;
begin
  nt1.pMulDiv(nt2, nt1);
  vt1.pMulDiv(vt2, vt1);
  AssertTrue('Vector pMulDivs do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestNumerics.TestAngleCosine;
begin
  Fs1 := nt1.AngleCosine(nt2);
  Fs2 := vt1.AngleCosine(vt2);
  AssertTrue('Vector AngleCosines do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2));
end;

{.$ifndef USE_ASM_AVX}
procedure TTestNumerics.TestAngleBetweem;
begin
  Fs1 := nt1.AngleBetween(nt2, norg);
  Fs2 := vt1.AngleBetween(vt2, vorg);
  AssertTrue('Vector AngleBetweens do not match'+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2));
end;
{.$endif}

initialization
  RegisterTest(TTestNumerics);

end.




