unit vectorother;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry,
  native, GLZVectorMath;

type

  { TTestVectorOther }

  TTestVectorOther = class(TTestCase)
    protected
      procedure Setup; override;
    public
     {$CODEALIGN RECORDMIN=16}
     vt1,vt2, vt3 : TGLZVector4f;
     at1, at2, at3, at4: TGLZVector;
     nt1,nt2, nt3 : TNativeGLZVector4f;
     ant1,ant2, ant3, ant4 : TNativeGLZVector4f;
     {$CODEALIGN RECORDMIN=4}
     Fs1,Fs2 : Single;
     nb, vb: boolean;

    published
      procedure TestCompare;       // these two test ensure we have data to play
      procedure TestCompareFalse;  // with and tests the compare function.
      procedure TestMinXYZComponent;
      procedure TestMaxXYZComponent;
      procedure TestMinVector;
      procedure TestMaxVector;
      procedure TestMinSingle;
      procedure TestMaxSingle;
      procedure TestClampVector;
      procedure TestClampSingle;
      procedure TestLerp;
      procedure TestPerp;
      procedure TestReflect;
//      procedure TestMoveAround;
      procedure TestCombine;
      procedure TestCombine2;
      procedure TestCombine3;

  end;

implementation

procedure TTestVectorOther.Setup;
begin
  vt1.Create(5.850,-15.480,8.512,1.5);
  vt2.Create(1.558,6.512,4.525,1.0);
  nt1.V := vt1.V;
  nt2.V := vt2.V;
  Fs1 := 1.5;
  Fs2 := 5.5;
end;

procedure TTestVectorOther.TestCompare;
begin
  AssertTrue('Test Values do not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt1));
end;

procedure TTestVectorOther.TestCompareFalse;
begin
  AssertFalse('Test Values should not match : '+nt1.ToString+' --> '+vt1.ToString, Compare(nt1,vt2));
end;

procedure TTestVectorOther.TestMinXYZComponent;
begin
  Fs1 := nt1.MinXYZComponent;
  Fs2 := vt1.MinXYZComponent;
  AssertTrue('Vector MinXYZComponents do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2));
end;

procedure TTestVectorOther.TestMaxXYZComponent;
begin
  Fs1 := nt1.MaxXYZComponent;
  Fs2 := vt1.MaxXYZComponent;
  AssertTrue('Vector MaxXYZComponents do not match : '+FLoattostrF(fs1,fffixed,3,3)+' --> '+FLoattostrF(fs2,fffixed,3,3), IsEqual(Fs1,Fs2));
end;

procedure TTestVectorOther.TestMinVector;
begin
  nt3 := nt1.Min(nt2);
  vt3 := vt1.Min(vt2);
  AssertTrue('Vector Min vectors do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestMaxVector;
begin
  nt3 := nt1.Max(nt2);
  vt3 := vt1.Max(vt2);
  AssertTrue('Vector Max vectors do not match'+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestMinSingle;
begin
  nt3 := nt1.Min(Fs1);
  vt3 := vt1.Min(Fs1);
  AssertTrue('Vector Min Singles do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestMaxSingle;
begin
  nt3 := nt1.Max(Fs1);
  vt3 := vt1.Max(Fs1);
  AssertTrue('Vector Max Singles do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestClampVector;
begin
  nt3 := nt1.Clamp(nt2,nt1);
  vt3 := vt1.Clamp(vt2,vt1);
  AssertTrue('Vector Clamp vectors do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));

end;

procedure TTestVectorOther.TestClampSingle;
begin
  nt3 := nt1.Clamp(fs2,fs1);
  vt3 := vt1.Clamp(fs2,fs1);
  AssertTrue('Vector Clamp vectors do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestLerp;
begin
  nt3 := nt1.Lerp(nt1,0.8);
  vt3 := vt1.Lerp(vt1,0.8);
  AssertTrue('Vector Clamp vectors do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));

end;

procedure TTestVectorOther.TestPerp;
begin
  nt3 := nt1.Perpendicular(nt2);
  vt3 := vt1.Perpendicular(vt2);
  AssertTrue('Vector Perpendiculars do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3, 1e-4));

end;

procedure TTestVectorOther.TestReflect;
begin
  nt3 := nt1.Reflect(nt2);
  vt3 := vt1.Reflect(vt2);
  AssertTrue('Vector Reflects do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3, 1e-4));
end;

procedure TTestVectorOther.TestCombine;
begin
  nt3 := nt1.Combine(nt2, fs1);
  vt3 := vt1.Combine(vt2, fs1);
  AssertTrue('Vector Combines do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestCombine2;
begin
  nt3 := nt1.Combine2(nt2, fs1, fs2);
  vt3 := vt1.Combine2(vt2, fs1, fs2);
  AssertTrue('Vector Combine2s do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

procedure TTestVectorOther.TestCombine3;
begin
  nt3 := nt1.Combine3(nt2, nt1, fs1, fs2, fs2);
  vt3 := vt1.Combine3(vt2, vt1, fs1, fs2, fs2);
  AssertTrue('Vector Combine3s do not match : '+nt3.ToString+' --> '+vt3.ToString, Compare(nt3,vt3));
end;

{
procedure TTestVectorOther.TestMoveAround;
begin
  nt3 := nt1.m(nt2);
  vt3 := vt1.Reflect(vt2);
  AssertTrue('Vector Reflects do not match', Compare(nt3,vt3));

end;
}

initialization
  RegisterTest(TTestVectorOther);

end.

